# (C) Copyright 2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (C) 2003 CGEY <http://cgey.com>
# Copyright (c) 2003 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: createVocabulariesFromCSV.py,v 1.1.1.1 2004/03/12 10:33:34 anguenot Exp $

"""Import and create vocabularies from CSV

There are files within the data sub-directories containing some lists of sender
types. They need to be store as vocabularies.  First step is to create the dict
structure (as in getMailFolderVocabularies)

How to use it ?

 - Log into the ZMI as manager
 - Go to your CPS root directory
 - Create an External Method with the following parameters:

     id    : Mail Manager Senter Type Import (or whatever)
     title : Mail Manager Sender Type Import  (or whatever)
     Module Name   :  CPSMailManager.createVocabulariesFormCSV
     Function Name : createVocabulariesFromCSV

 - save it
 - click now the test tab of this external method.
"""

import os, sys
import string
from zLOG import LOG, INFO, DEBUG

_log = []

LISTES = ['Conseil-regionaux',
          'Departements',
          'Prefectures',
          'Deputes1Col',
          'Directions',
          'Ministeres',
          'Senateurs1Col',
          'Sous-Prefectures']

#
# MAIN CALL
#

def createVocabulariesFromCSV(self):
    """Function mail call to create the vocabularies lists.
    """

    portal = self.portal_url.getPortalObject()

    def pr(bla, _log=_log):
        if bla == 'flush':
            return '<html><head><title>CPS Mail Manager CSV IMPORT</title></head><body><pre>'+ \
                   '\n'.join(_log) + \
                   '</pre></body></html>'
        _log.append(bla)
        if (bla):
            LOG('CPS Mail Manager CSV IMPORT:', INFO, bla)

    def primp(pr=pr):
        pr(" !!! Cannot migrate that component !!!")

    def prok(pr=pr):
        pr(" Already correctly installed")

    def portalhas(id, portal=portal):
        return id in portal.objectIds()

    vocabularies = {}
    vtool = portal.portal_vocabularies
    for list_name in LISTES:
        f = open(INSTANCE_HOME+'/Products/CPSMailManager/Data/'+list_name+'.csv',
                 'r').readlines()
        list = []
        for elt in f:
            list.append(elt)
        dict = {}
        for elt in list:
            dict[elt] = elt
        vocabularies[list_name] = {'data': {'dict': dict, 'list':list}}

    for id, info in vocabularies.items():
        pr(" Vocabulary %s" % id)
        if id in vtool.objectIds():
            pr("  Deleting.")
            vtool.manage_delObjects([id])
        pr("  Installing.")
        type = info.get('type', 'CPS Vocabulary')
        vtool.manage_addCPSVocabulary(id, type, **info['data'])

    pr(repr(vocabularies))
    return pr("flush")
