# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: install.py,v 1.13 2004/05/07 12:18:20 anguenot Exp $

"""CPSMailManager Installer

HOWTO USE THAT ?

 - Log into the ZMI as manager
 - Go to your CPS root directory
 - Create an External Method with the following parameters:

     id    : Mail Manager Installer (or whatever)
     title : Mail Manager Installer (or whatever)
     Module Name   :  CPSMailManager.install
     Function Name : install

 - save it
 - click now the test tab of this external method.
"""

from zLOG import LOG, INFO, DEBUG

import os, sys

from Products.PythonScripts.PythonScript import PythonScript

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent, View

from Products.CPSInstaller.CPSInstaller import CPSInstaller

from Products.CPSCore.CPSWorkflow import \
     TRANSITION_INITIAL_PUBLISHING, TRANSITION_INITIAL_CREATE, \
     TRANSITION_ALLOWSUB_CREATE, TRANSITION_ALLOWSUB_PUBLISHING, \
     TRANSITION_BEHAVIOR_PUBLISHING, TRANSITION_BEHAVIOR_FREEZE, \
     TRANSITION_BEHAVIOR_DELETE, TRANSITION_BEHAVIOR_MERGE, \
     TRANSITION_ALLOWSUB_CHECKOUT, TRANSITION_INITIAL_CHECKOUT, \
     TRANSITION_BEHAVIOR_CHECKOUT, TRANSITION_ALLOW_CHECKIN, \
     TRANSITION_BEHAVIOR_CHECKIN, TRANSITION_ALLOWSUB_DELETE, \
     TRANSITION_ALLOWSUB_MOVE, TRANSITION_ALLOWSUB_COPY


from Products.CPSMailManager.CPSMailManagerPermissions import MailCenterView, \
     ArchivesMailCenterView

SECTIONS_ID = 'sections'
WORKSPACES_ID = 'workspaces'
MAILS_ID = 'courriers'

SKINS = {
    'mail_default_scripts' : 'Products/CPSMailManager/skins/mail_default_scripts',
    'mail_default_templates' : 'Products/CPSMailManager/skins/mail_default_templates',
    'mail_content' :  'Products/CPSMailManager/skins/mail_content',
    'mail_libs': 'Products/CPSMailManager/skins/mail_libs',
    'mail_images': 'Products/CPSMailManager/skins/mail_images',
    'mail_styles':'Products/CPSMailManager/skins/mail_styles',
    'mail_directory': 'Products/CPSMailManager/skins/mail_directory',
    'mail_forum': 'Products/CPSMailManager/skins/mail_forum',
    }


class CPSMailManagerInstaller(CPSInstaller):
    """ Installer class for CPS Mail Manager component

    Intended to be use as an Installer/Updater tool.
    """

    product_name = 'CPSMailManager'

    def install(self):
        """ Installs the compulsory elements.

        Calling feature methods.
        """

        self.log("Starting CPSMailManager install")
        self.verifySkins(SKINS)
        self.resetSkinCache()
        self.installMailFolderTypeWorkflows()
        self.installRootMailFolderWorkflow()
        self.installMailTypeWorkflows()
        self.setupSpecificRoles()
        self.installWidgets()
        self.installSchemas()
        self.installLayouts()
        self.installVocabularies()
        self.installMailFolderTypeWorkflows()
        self.installMailRootFolder()
        self.setupPortalTypes()
        self.setupTranslations()
        self.createVocabulariesFromCSV()
        self.updateWorkflowAssociationsForMisillDocuments()
        self.installMailManagerBoxes()
        self.installCPSActions()
        self.installSubscriptionsFolder()
        self.extendCPSWorkspaceWorkflow()
        self.setupCPSMailManagerPermissions()
        self.finalize()
        self.reindexCatalog()
        self.log("End of specific CPSMailManager install")

    def installSchemas(self):
        """
        Install Specfic Schemas
        """

        self.log("Verifiying schemas")

        #
        # Mail / Mail Folder content types
        #

        schemas_mail_folder = self.portal.getMailFolderSchemas()
        schemas_mail = self.portal.getMailSchemas()

        schemas = schemas_mail_folder
        for key in schemas_mail.keys():
            schemas[key] = schemas_mail[key]

        stool = self.portal.portal_schemas
        for id, info in schemas.items():
            self.log(" Schema %s" % id)
            if id in stool.objectIds():
                self.log("  Deleting.")
                stool.manage_delObjects([id])
            self.log("  Installing.")
            schema = stool.manage_addCPSSchema(id)
            for field_id, fieldinfo in info.items():
                self.log("   Field %s." % field_id)
                schema.manage_addField(field_id, fieldinfo['type'],
                                       **fieldinfo['data'])

    def installLayouts(self):
        """
        Install the spcific layouts
        """

        self.log("Verifiying layouts")

        #
        # Mail / Mail Folder content types
        #

        layouts_mail_folder = self.portal.getMailFolderLayouts()
        layouts_mail = self.portal.getMailLayouts()

        layouts = layouts_mail_folder

        for key in layouts_mail.keys():
            layouts[key] = layouts_mail[key]

        ltool = self.portal.portal_layouts
        for id, info in layouts.items():
            self.log(" Layout %s" % id)
            if id in ltool.objectIds():
                self.log("  Deleting.")
                ltool.manage_delObjects([id])
            self.log("  Installing.")
            layout = ltool.manage_addCPSLayout(id)
            for widget_id, widgetinfo in info['widgets'].items():
                self.log("   Widget %s" % widget_id)
                widget = layout.manage_addCPSWidget(widget_id, widgetinfo['type'],
                                                    **widgetinfo['data'])
                layout.setLayoutDefinition(info['layout'])
                layout.manage_changeProperties(**info['layout'])

    def installVocabularies(self):
        """
        Install the specific vocabularies
        """

        self.log("Verifiying vocabularies")
        vocabularies = self.portal.getMailFolderVocabularies()

        vtool = self.portal.portal_vocabularies
        for id, info in vocabularies.items():
            self.log(" Vocabulary %s" % id)
            if id in vtool.objectIds():
                self.log("  Deleting.")
                vtool.manage_delObjects([id])
            self.log("  Installing.")
            type = info.get('type', 'CPS Vocabulary')
            vtool.manage_addCPSVocabulary(id, type, **info['data'])

    def installWidgets(self):
        """
        Install the specific widgets
        """
        self.log("Verifying widgets")
        widgets = self.portal.getMailFolderWidgets()

        wtool = self.portal.portal_widget_types
        for id, info in widgets.items():
            self.log(" Widget %s" % id)
            if id in wtool.objectIds():
                self.log("  Deleting.")
                wtool.manage_delObjects([id])
            self.log("  Installing.")
            widget = wtool.manage_addCPSWidgetType(id, info['type'])
            widget.manage_changeProperties(**info['data'])

    def setupSpecificRoles(self):
        """ Install CPSMailManager specific roles.

        Right now a specific role is needed to prevent Members to
        have the view permission on other mail folders than the ones
        on which they are actors.
        """

        new_roles = ['CPSMailManagerActor',
                     ]

        already = self.portal.valid_roles()

        for new_role in new_roles:
            if new_role not in already:
                self.portal._addRole(new_role)
                self.log(" Add role for mail manager %s" % new_role)


    def installMailFolderTypeWorkflows(self):
        """MailFolder Workflows Installation
        """

        #
        # InComingMailFolder workflow
        #

        from Products.CPSMailManager.Workflows.InComingMailFolderWorkflow import \
             in_coming_mailfolder_wf_install
        in_coming_mailfolder_wf_install(self.context)

        #
        # SpontaneousMailFolder workflow
        #

        from Products.CPSMailManager.Workflows.SpontaneousMailFolderWorkflow import \
            spontaneousMailFolderWorkflowInstall
        spontaneousMailFolderWorkflowInstall(self.context)

    def installRootMailFolderWorkflow(self):
         """Root Mail Folder workflow installation

         This is a workspace_folder_wf cps default one with different
         permissions settings
         """
         from Products.CPSMailManager.Workflows.RootMailFolderWorkflow import \
              rootMailFolderWorkflowInstall
         rootMailFolderWorkflowInstall(self.context)

    def createVocabulariesFromCSV(self):
        """
        Create vocabularies from CVS
        """
        try:
            from Products.CPSMailManager.Extensions.createVocabulariesFromCSV import \
                 createVocabulariesFromCSV
            createVocabulariesFromCSV(self.context)
        except (IOError, ):
            pass

    def installMailTypeWorkflows(self):
        """
        Mail content type workflows.
        """
        from Products.CPSMailManager.Workflows.MailContentTypeWorkflow import \
             mail_content_wf_install
        mail_content_wf_install(self.context)


    def allowTypesInMailRootFolder(self):
        """
        Allow types within the RootMailFolderType.
        """

        if 'RootMailFolder' in self.ttool.objectIds():
            mailsrootfolder_act = list(self.ttool['Workspace'].allowed_content_types)
        else:
            raise "DependanceError", 'RootMailFolder'
        for ptype in self.newptypes:
            if ptype not in mailsrootfolder_act:
                mailsrootfolder_act.append(ptype)
        self.ttool['RootMailFolder'].allowed_content_types = mailsrootfolder_act

    def allowTypesInMailFolderTypes(self):
        """
        Allow flex types within the Mail Folder Types
        """

        for container in ('InComingMailFolder',
                          'SpontaneousMailFolder',):
            if container in self.ttool.objectIds():
                containerACT = list(self.ttool['Workspace'].allowed_content_types)
            else:
                raise "DependanceError", container
            for ptype in self.newptypes:
                if ptype not in  containerACT:
                    containerACT.append(ptype)
                    allowed_content_type = {
                        container : containerACT,
                        }
                    self.ttool[container].allowed_content_types = allowed_content_type[container]

    def registerTypes(self):
        """
        Portal Types registration.
        """

        ptypes_installed = self.ttool.objectIds()
        display_in_cmf_calendar = []
        for ptype, data in self.flextypes.items():
            self.log("  Type '%s'" % ptype)
            if ptype in ptypes_installed:
                self.ttool.manage_delObjects([ptype])
                self.log("   Deleted")
            ti = self.ttool.addFlexibleTypeInformation(id=ptype)
            if data.get('display_in_cmf_calendar'):
                display_in_cmf_calendar.append(ptype)
                del data['display_in_cmf_calendar']
            ti.manage_changeProperties(**data)
            self.log("   Installation")

            # register ptypes to portal_calendar
            if display_in_cmf_calendar and hasattr(self.portal, 'portal_calendar'):
                self.portal.portal_calendar.calendar_types = display_in_cmf_calendar

    def updateMailsPortalTree(self):
        """
        Updating of  the portal tree
        """
        folderish_mail_folder_types = ['InComingMailFolder',
                                       'SpontaneousMailFolder',]

        self.log("Registering folderish document types in portal_tree")
        trtool = self.portal.portal_trees
        trtool[MAILS_ID].manage_changeProperties(
            type_names=list(trtool[MAILS_ID].type_names)
            + folderish_mail_folder_types)
        trtool[MAILS_ID].manage_rebuild()

    def updateWorkflowAssociations(self, default="workspace_folder_wf"):
        """
        Check workflow association
         """
        self.log("Verifying local workflow association")

        courriers = self.portal[MAILS_ID]

        if not '.cps_workflow_configuration' in courriers.objectIds():
            courriers.manage_addProduct['CPSCore'].addCPSWorkflowConfiguration()
            wfc = getattr(courriers, '.cps_workflow_configuration')
        else:
            wfc = getattr(courriers, '.cps_workflow_configuration')

        for ptype in self.newptypes:
            if 'cps_workspace_wf' in self.flextypes[ptype].keys():
                wwf = self.flextypes[ptype]['cps_workspace_wf']
            else:
                wwf = default
            self.log("  Add %s chain to portal type %s in %s of %s" % (
                wwf, ptype, '.cps_workflow_configuration', MAILS_ID))
            wfc.manage_addChain(portal_type=ptype, chain=wwf)


    def updateWorkflowAssociationsForMisillDocuments(self):
        """ Allowed the creation of MISILL documents within the
        mailfolder
        """

        courriers = self.portal[MAILS_ID]

        if not '.cps_workflow_configuration' in courriers.objectIds():
            courriers.manage_addProduct['CPSCore'].addCPSWorkflowConfiguration()
            wfc = getattr(courriers, '.cps_workflow_configuration')
        else:
            wfc = getattr(courriers, '.cps_workflow_configuration')

        newtypes = self.portal.portal_types['Workspace'].allowed_content_types
        for ptype in newtypes:
            if ptype in ['CPSChat', 'CPSForum', 'Calendar', 'Workspace']:
                wwf = 'workspace_folder_wf'
            elif ptype in ['Event', 'EventDoc', 'FAQitem', 'File', 'Flexible','GlossaryItem', 'Image', 'Link', 'News']:
                wwf = 'workspace_content_wf'
            elif ptype in ['ImageGallery', 'Glossary' ,'FAQ']:
                wwf = 'workspace_folderish_content_wf'
            else:
                wwf = ''
            if wwf:
                wfc.manage_addChain(portal_type=ptype, chain=wwf)


    def changeActions(self, types=None):
        """
        Change Actions for flex types
        """
        #
        # TO FIX !!!!!
        # listActions dosen't work... how come ?
        # Change de hack in here for the actions cause
        # it's pretty messy.
        #
        ttool = self.portal.portal_types
        if types == "mailfolder":
           for type in self.newptypes:
               ttype = self.ttool[type]
               ttype.deleteActions((0, 1, 2, 3, 4, 5, 6))

               ttype.addAction('view',
                               'action_summary',
                               'mailfolder_view',
                               '',
                               'View',
                               'object')
               ttype.addAction('comments',
                               'action_comments',
                               'mailfolder_comments',
                               '',
                               'View',
                               'object'),
               ttype.addAction('work_document',
                               'action_work_document',
                               'mail_contents',
                               '',
                               'View',
                               'object')
               ttype.addAction('work_history',
                               'action_history',
                               'mailfolder_history',
                               '',
                               'View',
                               'object')
               ttype.addAction('work_people',
                               'action_people',
                               'mailfolder_people',
                               '',
                               'View',
                               'object')
               ttype.addAction('action_metadata_view',
                               'action_metadata_view',
                               'mailfolder_edit_form',
                               '',
                               'View',
                               'object')

               self.log("Changing actions for type %s" %ttype)

        elif types == "mailtype":
            ttype = ttool['mail_document']
            ttype.deleteActions((0, 1, 2, 3, 4, 5))

            ttype.addAction('view',
                            'action_document',
                            'string:${object_url}/mailtype_view',
                            '',
                            'View',
                            'object')
            ttype.addAction('edit',
                            'action_edit',
                            'string:${object_url}/mailtype_edit_form',
                            '',
                            'Modify portal content',
                            'object')
            ttype.addAction('modify',
                            'action_modify',
                            'string:${object_url}/mailtype_edit_form',
                            'python:folder.canModifyReferenceDocument() and (object.REQUEST["URL0"].find("mailtype_view")>-1 or object.REQUEST["URL0"].find("view")>-1)',
                            'Modify portal content',
                            'object_manage')
            ttype.addAction('action_metadata_view',
                            'action_metadata_view',
                            'string:${object_url}/mailtype_edit_metadata_form',
                            'python:folder.canModifyReferenceDocument()',
                            'Modify portal content',
                            'object')
            ttype.addAction('comments',
                            'action_comments',
                            'mailtype_comments',
                            '',
                            'View',
                            'object'),
            ttype.addAction('versions',
                            'action_versions',
                            'mailtype_versions',
                            '',
                            'View',
                            'object')
            ttype.addAction('work_history',
                            'action_history',
                            'mailtype_history',
                            '',
                            'View',
                            'object')

        #
        # Removing the status  history action from the portal action
        #
        # FIXME : not for all document types like in MISILL !
        #

        self.log("Removing the status_history_action from the portal actions")
        actiondelmap = {
            'portal_actions': ('status_history', ),
            }
        for tool, actionids in actiondelmap.items():
            actions = list(self.portal[tool]._actions)
            for ac in actions:

                id = ac.id
                if id in actionids:
                    if ac.visible:
                        ac.visible = 0
                    self.log(" Deleting %s: %s" % (tool, id))
            self.portal[tool]._actions = actions

    def setupPortalTypes(self):
        """
        Types for the CPSMailManager
        """

        self.ttool = self.portal.portal_types

        self.log("Verifying Mail Folder types for the CPSMailManager")
        self.flextypes = self.portal.getMailFolderTypes()
        self.newptypes = self.flextypes.keys()
        self.allowTypesInMailRootFolder()
        self.registerTypes()
        self.changeActions(types="mailfolder")
        self.updateWorkflowAssociations()

        self.log("Verifying Mail Content Types for the CPSMailManager")
        self.flextypes = self.portal.getMailTypes()
        self.newptypes = self.flextypes.keys()
        self.allowTypesInMailFolderTypes()
        self.registerTypes()
        self.changeActions(types="mailtype")
        self.updateWorkflowAssociations(default="workspace_content_wf")

    def setupRootMailFolderType(self):
        """
        Root Mail Folder Type setup
        """

        self.ttool = self.portal.portal_types
        ptypes_installed = self.ttool.objectIds()
        self.log("Verifying portal types for the CPSMailManager")
        self.ptypes = {
            'CPSMailManager' : ('RootMailFolder',
                                   ),
            }
        for prod in self.ptypes.keys():
            for ptype in self.ptypes[prod]:
                self.log("  Type '%s'" % ptype)
                if ptype in ptypes_installed:
                    self.ttool.manage_delObjects([ptype])
                    self.log("   Deleted")
                self.ttool.manage_addTypeInformation(
                    id=ptype,
                    add_meta_type='Factory-based Type Information',
                    typeinfo_name=prod+': '+ptype
                    )

        #
        # Check workflow association
        #
        self.log("Verifying workflow schemas association for RootMailFolder")
        wfs = {
            'RootMailFolder': 'root_mail_folder_wf',
            }

        wftool = self.portal.portal_workflow

        self.log("Installing workflow schemas for RootMailFolder")
        for pt, chain in wfs.items():
            wftool.setChainForPortalTypes([pt], chain)

    def setupRootMailFolderPermissions(self):
        """
        Permissions on the MailRootFolder
        """

        from Products.CMFCore.CMFCorePermissions import setDefaultRoles
        ModifyFolderPoperties = 'Modify Folder Properties'
        setDefaultRoles( ModifyFolderPoperties, ( 'Manager',))

        courriers_perm = {
            'Add portal content': ['Manager', 'WorkspaceManager', 'WorkspaceMember'],
            'Add portal folders': ['Manager', 'WorkspaceManager',],
            'Change permissions': ['Manager', 'WorkspaceManager'],
            'Delete objects': ['Manager', 'WorkspaceManager', 'WorkspaceMember', ],
            'List folder contents': ['Manager', 'WorkspaceManager', 'WorkspaceMember','WorkspaceReader', 'Member'],
            'Modify portal content': ['Manager', 'WorkspaceManager', 'WorkspaceMember','Member'],
            'Modify Folder Properties' : ['Manager', 'WorkspaceManager'],
            'View': ['Manager', 'WorkspaceManager', 'WorkspaceMember', 'WorkspaceReader', 'Member'],
            'View management screens': ['Manager', 'WorkspaceManager', 'WorkspaceMember', 'Member'],
            'Add Box Container': ['Manager', 'WorkspaceManager', 'SectionManager'],
            'Manage Box Overrides': ['Manager','WorkspaceManager'],
            'Manage Boxes': ['Manager', 'WorkspaceManager'],
            }

        for perm, roles in courriers_perm.items():
            self.portal[MAILS_ID].manage_permission(perm, roles, 0)
            self.log("  Permission %s" % perm)

        # XXXX to check why ?
        # AttributeError: NoWorkflowConfiguration instance has no attribute 'getPhysicalPath'
        try:
            self.portal[MAILS_ID].reindexObjectSecurity()
        except (AttributeError, KeyError):
            pass

    def setupRootMailFolderTree(self):
        """
        Its own tree
        """

        trtool = self.portal.portal_trees
        self.log("Verifying cache trees")
        if MAILS_ID in trtool.objectIds():
            trtool.manage_delObjects([MAILS_ID])
        self.log("  Adding cache for tree %s" % MAILS_ID)
        trtool.manage_addCPSTreeCache(id=MAILS_ID)
        trtool[MAILS_ID].manage_changeProperties(
            title=MAILS_ID+' Cache',
            root=MAILS_ID,
            type_names=('RootMailFolder',
                        'InComingMailFolder',
                        'SpontaneousMailFolder',),
            meta_types=('CPS Proxy Folder',
                        'CPS Proxy Document',
                        'CPS Proxy Folderish Document',
                        'InComingMailFolder',
                        'SpontaneousMailFolder',),
            info_method='getFolderInfo')
        trtool[MAILS_ID].manage_rebuild()
        self.log("Mails Main Section cache rebuilded")

    def installMailRootFolder(self):
        """ Installs the root mail folder
        """

        self.setupRootMailFolderType()

        self.log("Verifying roots of : %s " % (MAILS_ID))
        if not hasattr(self.portal, MAILS_ID):
            #self.portal.manage_delObjects([MAILS_ID])
            self.portal.portal_workflow.invokeFactoryFor(self.portal.this(),
                                                         'RootMailFolder',
                                                         MAILS_ID)
        self.portal[MAILS_ID].getContent().setTitle('Courriers')
        self.log("  Adding %s RootFolder" % MAILS_ID)

        self.portal[MAILS_ID].reindexObject()
        self.setupRootMailFolderPermissions()

    def installMailManagerBoxes(self):
        """Install Mail Manager related Boxes

        Boxes registered by MisillDefault.  Use and xml export cause override
        on .cps_boxes_root and not really nice API at all to do in Python
        """

        boxes = {
            'courriers1712': {'type': 'Action Box',
                              'title': 'Courriers',
                              'provider': 'nuxeo',
                              'btype': 'default',
                              'box_skin': 'here/box_lib/macros/mmcbox',
                              'minimized': 0,
                              'closed': 0,
                              'slot': 'left',
                              'order': 100,
                              'display_in_subfolder': 1,
                              'display_only_in_subfolder': 0,
                              'locked': 0,
                              'categories': ('courriers',),
                              },
            }

        self.verifyBoxes(boxes)

    def installCPSActions(self):
        """Install CPS actions
        """

        #
        # Cleaning actions
        #

        actiondelmap = {
            'portal_actions': ('mail_center',
                               'archives_mail_center',)}

        for tool, actionids in actiondelmap.items():
            actions = list(self.portal[tool]._actions)
            new_actions = []
            for ac in actions:
                id = ac.id
                if id not in actionids:
                    new_actions.append(ac)
                self.portal[tool]._actions = new_actions

        #
        # ACTION : Mails center
        # category :Portal
        #

        self.portal['portal_actions'].addAction(
            id='mail_center',
            name='action_mail_center',
            action='string: ${portal_url}/courriers',
            condition="",
            permission=(MailCenterView,),
            category='courriers',
            visible=1)
        self.log("  Added Action Mails center  ")

        #
        # ACTION : Mails center
        # category :Portal
        #

        self.portal['portal_actions'].addAction(
            id='archives_mail_center',
            name='action_archives_mail_center',
            action='string: ${portal_url}/courriers?filter_flag=closed',
            condition="",
            permission=(ArchivesMailCenterView,),
            category='courriers',
            visible=1)

        self.log(" Added Action Archives Mails center  ")

    def installSubscriptionsFolder(self):
        """Install subscriptions Folder with mail__notification_rule object
        """

        #
        # Subscriptions folder
        #

        root_mail_folder = self.portal[MAILS_ID]
        if not hasattr(root_mail_folder, '.cps_subscriptions'):
            root_mail_folder.manage_addProduct['CPSSubscriptions'].addSubscriptionContainer()
            subscriptions_folder = getattr(root_mail_folder,
                                           self.portal.portal_subscriptions.getSubscriptionContainerId())
            subscriptions_folder.manage_addProduct['CPSSubscriptions'].addMailNotificationRule(
                id='mail__notification_rule')

        #
        # Now message content
        #

        subscriptions_tool = getToolByName(self.portal, 'portal_subscriptions')
        try:
            mail_manager_events = self.portal.getMailManagerMessages()
            for mail_manager_event in mail_manager_events:
                subscriptions_tool.manage_editEventMessage(mail_manager_event['id'],
                                                           mail_manager_event['title'],
                                                           mail_manager_event['body'],
                                                           '',
                                                           '',
                                                           '',
                                                           '',
                                                           '',
                                                           '',
                                                           '',
                                                           '',)
        except (AttributeError,):
            pass

    def extendCPSWorkspaceWorkflow(self):
        """Extends the CPS default workflow

        Add new_version / modification / lock transitions.
        """

        wftool = self.portal.portal_workflow
        try:
            wf = wftool['workspace_content_wf']
        except KeyError:
            raise "CPSDefault dependencies.: You must install CPSDefault if you want to \
            use CPSMailManager"

        try:
            wf.transitions.addTransition('new_version')
            wf.transitions.addTransition('modification')
            wf.transitions.addTransition('lock')
        except:
            # XXX
            pass

        s = wf.states.get('locked')
        s.setProperties(transitions=('new_version', 'modification', 'unlock',))
        s.setPermission(ModifyPortalContent, 0, ('MailTypeLocker', 'Manager'))

        s = wf.states.get('work')
        transitions = s.transitions
        s.setProperties(transitions=transitions + ('lock', 'new_version', 'modification'))

        t = wf.transitions.get('new_version')
        t.setProperties(title='Nouvelle version du document', new_state_id='work',
                        transition_behavior=(TRANSITION_BEHAVIOR_FREEZE, ),
                        clone_allowed_transitions=None,
                        actbox_name='', actbox_category='workflow', actbox_url='',
                        after_script_name = 'unlock',
                        props={'guard_permissions':'',
                               'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; ',
                               'guard_expr':''},
                        )

        t = wf.transitions.get('modification')
        t.setProperties(title='Modification du document', new_state_id='work',
                        transition_behavior=(),
                        clone_allowed_transitions=None,
                        actbox_name='', actbox_category='workflow', actbox_url='',
                        after_script_name = 'unlock',
                        props={'guard_permissions':'',
                               'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; ',
                               'guard_expr':''},
                        )

        t = wf.transitions.get('lock')
        t.setProperties(title='Vouillage du document', new_state_id='locked',
                        transition_behavior=(),
                        clone_allowed_transitions=None,
                        actbox_name='', actbox_category='workflow', actbox_url='',
                        after_script_name = 'lock',
                        props={'guard_permissions':'',
                               'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; ',
                               'guard_expr':''},
                        )

        t = wf.transitions.get('checkout_draft')
        t.setProperties(title='Checkout content into a draft',
                    new_state_id='locked',
                    transition_behavior=(TRANSITION_BEHAVIOR_CHECKOUT,),
                    checkout_allowed_initial_transitions=('checkout_draft_in',),
                    actbox_name='action_checkout_draft', actbox_category='workflow',
                    actbox_url='%(content_url)s/content_checkout_draft_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; WorkspaceMember',
                           'guard_expr':'python:hasattr(here, "portal_type") and here.portal_type != "mail_document"'},
                        )

        # wf scripts
        scripts = wf.scripts

        script_name = 'lock'
        try:
            scripts.manage_delObjects(['lock'])
        except:
            pass
        scripts._setObject(script_name, PythonScript(script_name))
        script = scripts[script_name]
        script.write("""\
##parameters=state_change
return state_change.object.mailtype_lock()
""")
        #script._proxy_roles = ('Manager',)
        script._owner = None

        script_name = 'unlock'
        try:
            scripts.manage_delObjects(['unlock'])
        except:
            pass
        scripts._setObject(script_name, PythonScript(script_name))
        script = scripts[script_name]
        script.write("""\
##parameters=state_change
return state_change.object.mailtype_unlock()
""")
        #script._proxy_roles = ('Manager',)
        script._owner = None

    def setupCPSMailManagerPermissions(self):
        """Setup specific permissions
        """

        self.log("Install CPSMailManager specific permissions")

        self.setupPortalPermissions({
            MailCenterView : ['Manager',
                              'Member',
                              ],
            ArchivesMailCenterView : ['Manager',
                                      'Member',
                                      ],
            })

        mail_manager_perms = {
            MailCenterView: ['Manager', 'Member',],
            ArchivesMailCenterView: ['Manager', 'Member',],
            }

        for perm, roles in mail_manager_perms.items():
            self.portal.manage_permission(perm, roles, 0)
            self.log("  Adding Permission %s  " % perm)

###############################################
# __call__
###############################################

def install(self):
    """Installation is done here.

    Called by an external method for instance.
    """

    installer = CPSMailManagerInstaller(self)
    installer.install()
    return installer.logResult()
