# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: MailManagerWidgets.py,v 1.2 2004/05/07 12:18:20 anguenot Exp $

__author__ = "Julien Anguenot <ja@nuxeo.com>"

"""
Misill Mail Manager widgets

Definition of widget types needed for the Mail Folder types
"""

from zLOG import LOG, DEBUG
from Globals import InitializeClass

from Products.CPSSchemas.BasicWidgets import CPSSelectWidget, \
     CPSSelectWidgetType, CPSLinesWidget, CPSLinesWidgetType, \
     renderHtmlTag
from Products.CPSSchemas.WidgetTypesTool import WidgetTypeRegistry

def _isinstance(ob, cls):
    try:
        return isinstance(ob, cls)
    except TypeError:
        # In python 2.1 isinstance() raises TypeError
        # instead of returning 0 for ExtensionClasses.
        return 0

##################################################

class MailManagerSenderTypeWidget(CPSSelectWidget):
    """ Sender type widget types."""

    _properties = CPSSelectWidget._properties + (
        {'id': 'destination_field', 'type': 'string', 'mode': 'w',
         'label': 'Destination Field'},
        )

    meta_type = "Mail Manager Sender Type Widget"

    def render(self, mode, datastructure, **kw):
        """Render in mode from datastructure."""

        if mode == 'view':
            return CPSSelectWidget.render(self, mode, datastructure)

        elif mode == 'edit':
            render_method = 'widget_mail_manager_sender_type_render'

            meth = getattr(self, render_method, None)
            if meth is None:
                raise RuntimeError("Unknown Render Method %s for widget type %s"
                                   % (render_method, self.getId()))
            return meth(mode=mode, datastructure=datastructure,
                        vocabulary=self._getVocabulary(datastructure))

        else:
            raise RuntimeError('unknown mode %s' % mode)

InitializeClass(MailManagerSenderTypeWidget)

class MailManagerSenderTypeWidgetType(CPSSelectWidgetType):
    """Mail Manager Sender Type widget type."""
    meta_type = "Mail Manager Sender Type Widget Type"
    cls = MailManagerSenderTypeWidget

InitializeClass(MailManagerSenderTypeWidgetType)

#######################################################

class MailManagerRecipientTypeWidget(CPSSelectWidget):
    """ Recipient type widget types."""

    _properties = CPSSelectWidget._properties + (
        {'id': 'destination_field', 'type': 'string', 'mode': 'w',
         'label': 'Destination Field'},
        )

    meta_type = "Mail Manager Recipient Type Widget"

    def render(self, mode, datastructure, **kw):
        """Render in mode from datastructure."""

        if mode == 'view':
            return CPSSelectWidget.render(self, mode, datastructure)

        elif mode == 'edit':
            render_method = 'widget_mail_manager_recipient_type_render'

            meth = getattr(self, render_method, None)
            if meth is None:
                raise RuntimeError("Unknown Render Method %s for widget type %s"
                                   % (render_method, self.getId()))
            return meth(mode=mode, datastructure=datastructure,
                        vocabulary=self._getVocabulary(datastructure))

        else:
            raise RuntimeError('unknown mode %s' % mode)

InitializeClass(MailManagerRecipientTypeWidget)

class MailManagerRecipientTypeWidgetType(CPSSelectWidgetType):
    """Mail Manager Recipient Type widget type."""
    meta_type = "Mail Manager Recipient Type Widget Type"
    cls = MailManagerRecipientTypeWidget

InitializeClass(MailManagerRecipientTypeWidgetType)


class MailManagerUserListWidget(CPSLinesWidget):
    """Mail Manager User List Widget

    Handles the contributor/pilot  lists.
    """

    meta_type = "Mail Manager User List Widget"

    def render(self, mode, datastructure, **kw):
        """Render in mode from datastructure."""

        stored_value = datastructure.getDataModel()[self.getWidgetId()]
        portal = self.portal_url.getPortalObject()

        #
        # Building here the list we gonna display
        # Let's try to get fullname given the ids.
        #

        value = []
        for member_id in stored_value:
            member_struct = portal.getEntryFromDirectories(member_id)
            from zLOG import LOG, DEBUG
            fullname = member_struct.get('givenName', '')\
                       + ' '\
                       + member_struct.get('sn', '')
            if fullname == ' ':
                fullname = member_struct.get('id', member_struct.get('uid', ''))
            value.append(fullname)

        return renderHtmlTag('textarea',
                             id=self.getHtmlWidgetId(),
                             name=self.getHtmlWidgetId()+":lines",
                             cols=self.width,
                             rows=self.height,
                             contents='\n'.join(value),
                             readonly=1)

InitializeClass(MailManagerUserListWidget)

class MailManagerUserListWidgetType(CPSLinesWidgetType):
    """Mail Manager User List Widget Type.
    """

    meta_type = "Mail Manager User List Widget Type"
    cls = MailManagerUserListWidget

InitializeClass(MailManagerUserListWidgetType)

##################################################

WidgetTypeRegistry.register(MailManagerSenderTypeWidgetType,
                            MailManagerSenderTypeWidget)
WidgetTypeRegistry.register(MailManagerRecipientTypeWidgetType,
                            MailManagerRecipientTypeWidget)
WidgetTypeRegistry.register(MailManagerUserListWidgetType,
                            MailManagerUserListWidget)

