# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: RootMailFolder.py,v 1.2 2004/05/07 12:18:20 anguenot Exp $

"""Folder, which is used as the root of mail folders.

We need it to redefine the action categories and to provide
a special view when accessing it.

As well, the global security access is done at global level (Member role)
and on each mail folder we'll apply local roles
"""

from zLOG import LOG, DEBUG

from types import StringType

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.BTreeFolder2.CMFBTreeFolder import CMFBTreeFolder

from Products.CMFCore.CMFCorePermissions import View
from Products.CPSCore.CPSBase import CPSBaseFolder, CPSBase_adder

factory_type_information = (
    {'id': 'RootMailFolder',
     'title': 'portal_type_Root_Mail_Folder_title',
     'description': 'portal_type_Root_Mail_Folder_description',
     'content_icon': 'root_mail_folder.gif',
     'product': 'CPSMailManager',
     'meta_type': 'RootMailFolder',
     'factory': 'addRootMailFolder',
     'immediate_view': 'folder_edit_form',
     'filter_content_types': 1,
     'allowed_content_types': ('InComingMailFolder',
                               'SpontaneousMailFolder',
                               'ParliamentMailFolder',
                               'ParliamantQuestionMailFolder',),
     'actions': ({'id': 'view',
                  'name': 'action_view',
                  'action': 'root_mailfolder_view',
                  'permissions': (View,)},),
     'cps_proxy_type': 'folder',
     },
    )


class RootMailFolder(CPSBaseFolder, CMFBTreeFolder):
    """This is the root mailfolder.

    All the mail folders of the portal are stored in here.
    """

    meta_type = 'RootMailFolder'
    portal_type = meta_type

    security = ClassSecurityInfo()

    def __init__(self, id, **kw):
        """ Constructor
        """

        CMFBTreeFolder.__init__(self, id)
        self.current_identifier_value = 0

    security.declareProtected(View, "getMailFolderIdentifier")
    def getMailFolderIdentifier(self):
        """Returns the last part of the mailfolder identifier

        The format of mailfolder identifier is as stated below :
        DGCL-YYYY-XXXXXX

        This is the XXXXXX part that we gonna return. It's incremented by one
        each time we create a mailfolder.
        """

        self.current_identifier_value += 1
        return self.current_identifier_value - 1

    security.declareProtected(View, "doDeleteMailFolders")
    def doDeleteMailFolders(self, ids=[], REQUEST=None):
        """Delete folder mailfolders given their ids
        Manager + pilote level 1 or allowed to do that.

        FIXME more security
        """

        portal = self.portal_url.getPortalObject()
        mails_repository = portal['courriers']
        if isinstance(ids, StringType):
            ids = [ids]
        mails_repository.manage_delObjects(ids)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(portal.absolute_url()+'/courriers')

InitializeClass(RootMailFolder)


def addRootMailFolder(container, id, REQUEST=None, **kw):
    """Add a Root Mail Folder.
    """
    ob = RootMailFolder(id, **kw)
    return CPSBase_adder(container, ob, REQUEST=REQUEST)
