# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: MailContentTypeWorkflow.py,v 1.1.1.1 2004/03/12 10:33:34 anguenot Exp $

"""Mail Content Type Workflow based on the standard workspace_content workflow
of CPS.

"""

import os, sys
from App.Extensions import getPath
from re import match
from zLOG import LOG, INFO, DEBUG
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from Products.PythonScripts.PythonScript import PythonScript

from Products.CPSCore.CPSWorkflow import \
     TRANSITION_INITIAL_CHECKOUT, \
     TRANSITION_INITIAL_CREATE, \
     TRANSITION_INITIAL_PUBLISHING, \
     TRANSITION_ALLOWSUB_CHECKOUT, \
     TRANSITION_ALLOWSUB_CREATE, \
     TRANSITION_ALLOWSUB_DELETE, \
     TRANSITION_ALLOWSUB_COPY, \
     TRANSITION_ALLOWSUB_MOVE, \
     TRANSITION_ALLOWSUB_PUBLISHING, \
     TRANSITION_ALLOW_CHECKIN, \
     TRANSITION_BEHAVIOR_CHECKOUT, \
     TRANSITION_BEHAVIOR_CHECKIN, \
     TRANSITION_BEHAVIOR_CHECKOUT, \
     TRANSITION_BEHAVIOR_CHECKIN, \
     TRANSITION_BEHAVIOR_DELETE, \
     TRANSITION_BEHAVIOR_FREEZE, \
     TRANSITION_BEHAVIOR_MERGE, \
     TRANSITION_BEHAVIOR_PUBLISHING

from Products.DCWorkflow.Transitions import \
     TRIGGER_AUTOMATIC, TRIGGER_USER_ACTION, TRIGGER_WORKFLOW_METHOD

def mail_content_wf_install(self):
    """
    Install the workflow for the InComingMailFolder Type
    """

    portal = self.portal_url.getPortalObject()
    wftool = portal.portal_workflow

    wfids = wftool.objectIds()
    wfid = 'mail_content_wf'

    if wfid in wfids:
        wftool.manage_delObjects([wfid])

    wftool.manage_addWorkflow(id=wfid,
                              workflow_type='cps_workflow (Web-configurable workflow for CPS)'
                              )
    wf = wftool[wfid]

    #
    # Initialization of Workflow components.
    #

    for s in ('work', 'draft', 'locked'):
        wf.states.addState(s)

    for t in ('create', 'copy_submit',
              'checkout_draft', 'checkout_draft_in', 'checkin_draft',
              'abandon_draft', 'unlock', 'cut_copy_paste'):
        wf.transitions.addTransition(t)

    for v in ('action', 'actor', 'comments', 'review_history', 'time',
              'dest_container'):
        wf.variables.addVariable(v)

    for p in (View, ModifyPortalContent, ):
        wf.addManagedPermission(p)

    s = wf.states.get('work')
    s.setProperties(title='Work',
                    transitions=('copy_submit', 'checkout_draft', 'cut_copy_paste'))
    s.setPermission(ModifyPortalContent, 0, ('Manager', 'WorkspaceManager', 'WorkspaceMember',
                                             'Owner', ))
    s.setPermission(View, 0, ('Manager', 'WorkspaceManager', 'WorkspaceMember', 'WorkspaceReader', 'Owner'))

    s = wf.states.get('draft')
    s.setProperties(title='Draft',
                    transitions=('checkin_draft', 'abandon_draft', 'unlock'))
    s.setPermission(ModifyPortalContent, 0, ('Manager', 'WorkspaceManager', 'Owner'))
    s.setPermission(View, 0, ('Manager', 'WorkspaceManager', 'Owner'))

    s = wf.states.get('locked')
    s.setProperties(title='Locked',
                    transitions=('unlock',))
    s.setPermission(ModifyPortalContent, 0, ())
    s.setPermission(View, 0, ('Manager', 'WorkspaceManager', 'WorkspaceMember', 'WorkspaceReader'))
    t = wf.transitions.get('create')
    t.setProperties(title='Initial creation', new_state_id='work',
                    transition_behavior=(TRANSITION_INITIAL_CREATE, ),
                    clone_allowed_transitions=None,
                    actbox_name='', actbox_category='workflow', actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; Member',
                           'guard_expr':''},
                    )
    t = wf.transitions.get('copy_submit')
    t.setProperties(title='Copy content into a section for Publishing',
                    new_state_id='',
                    transition_behavior=(TRANSITION_BEHAVIOR_PUBLISHING, ),
                    clone_allowed_transitions=('submit', 'publish'),
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_submit', actbox_category='workflow',
                    actbox_url='%(content_url)s/content_submit_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager',
                           'guard_expr':''},
                    )
    t = wf.transitions.get('checkout_draft')
    t.setProperties(title='Checkout content into a draft',
                    new_state_id='locked',
                    transition_behavior=(TRANSITION_BEHAVIOR_CHECKOUT,),
                    checkout_allowed_initial_transitions=('checkout_draft_in',),
                    actbox_name='action_checkout_draft', actbox_category='fake_object_manage',
                    actbox_url='%(content_url)s/mail_checkout_draft_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; Owner',
                           'guard_expr':''},
                    )
    t = wf.transitions.get('checkout_draft_in')
    t.setProperties(title='Draft is created',
                    new_state_id='draft',
                    transition_behavior=(TRANSITION_INITIAL_CHECKOUT,
                                         TRANSITION_BEHAVIOR_FREEZE),
                    )
    t = wf.transitions.get('checkin_draft')
    t.setProperties(title='Checkin draft',
                    new_state_id='locked',
                    transition_behavior=(TRANSITION_BEHAVIOR_CHECKIN,),
                    checkin_allowed_transitions=('unlock',),
                    actbox_name='action_checkin_draft', actbox_category='fake_object_manage',
                    actbox_url='%(content_url)s/mail_checkin_draft_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; Owner',
                           'guard_expr':''},
                    )
    t = wf.transitions.get('abandon_draft')
    t.setProperties(title='Abandon draft',
                    new_state_id='',
                    transition_behavior=(TRANSITION_BEHAVIOR_DELETE,),
                    script_name='unlock_locked_before_abandon',
                    actbox_name='action_abandon_draft', actbox_category='fake_object_manage',
                    actbox_url='%(content_url)s/mail_abandon_draft_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; Owner',
                           'guard_expr':''},
                    )
    t = wf.transitions.get('unlock')
    t.setProperties(title='Unlock content after a draft is done',
                    new_state_id='work',
                    transition_behavior=(TRANSITION_ALLOW_CHECKIN,),
                    )
    # For the cut/copy/paste feature
    t = wf.transitions.get('cut_copy_paste')
    t.setProperties(title='Cut/Copy/Paste', new_state_id='work',
                    transition_behavior=(TRANSITION_ALLOWSUB_DELETE,
                                         TRANSITION_ALLOWSUB_MOVE,
                                         TRANSITION_ALLOWSUB_COPY),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='New',
                    actbox_category='',
                    actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; ',
                           'guard_expr':''},
                    )
    # wf scripts
    scripts = wf.scripts

    script_name = 'unlock_locked_before_abandon'
    scripts._setObject(script_name, PythonScript(script_name))
    script = scripts[script_name]
    script.write("""\
##parameters=state_change
return state_change.object.content_unlock_locked_before_abandon(state_change)
""")
    #script._proxy_roles = ('Manager',)
    script._owner = None

    # wf variables
    wf.variables.setStateVar('review_state')
    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions':'',
                              'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; WorkspaceReader; Member',
                              'guard_expr':''})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)

    vdef = wf.variables['dest_container']
    vdef.setProperties(description='Destination container for the last paste/publish',
                       default_expr="python:state_change.kwargs.get('dest_container', '')",
                       for_status=1, update_always=1)
