# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (C) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: RootMailFolderWorkflow.py,v 1.2 2004/05/07 12:18:20 anguenot Exp $

"""Root Mail Folder Workflow based on the standard workspace_folder workflow of
CPS.

New permission setting for security purposes. The main difference is the use of
the global role Member instead of WorkspaceMember in the MISILL case.  No local
roles anymore using the 'role:Authenticated' and security checks done by the
workflow.
"""

import os, sys
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.CPSCore.CPSWorkflow import \
     TRANSITION_INITIAL_CREATE, \
     TRANSITION_ALLOWSUB_CREATE

from Products.DCWorkflow.Transitions import \
     TRIGGER_USER_ACTION

def rootMailFolderWorkflowInstall(self):
    """Installs the workflow for the RootMailFolder.
    """

    portal = self.portal_url.getPortalObject()
    wftool = portal.portal_workflow

    wfids = wftool.objectIds()
    wfid = 'root_mail_folder_wf'

    if wfid in wfids:
        wftool.manage_delObjects([wfid])

    wftool.manage_addWorkflow(id=wfid,
                              workflow_type='cps_workflow (Web-configurable workflow for CPS)')

    wf = wftool[wfid]

    for s in ('work', ):
        wf.states.addState(s)

    for t in ('create', 'create_content'):
        wf.transitions.addTransition(t)

    for p in (View, ModifyPortalContent, ):
        wf.addManagedPermission(p)

    ###########################################################################
    ###########################################################################

    #                                  STATES

    ###########################################################################
    ###########################################################################

    s = wf.states.get('work')


    s.setPermission(ModifyPortalContent, 0, ('Manager',
                                             'Member',
                                             'Owner',
                                             ))
    s.setPermission(View, 0, ('Manager',
                              'Member',
                              'Owner'
                              ))

    s.setProperties(title='Work',
                    transitions=('create_content',))


    ###########################################################################
    ###########################################################################

    #                               TRANSITIONS

    ###########################################################################
    ###########################################################################

    t = wf.transitions.get('create')
    t.setProperties(title='Initial creation', new_state_id='work',
                    transition_behavior=(TRANSITION_INITIAL_CREATE,),
                    clone_allowed_transitions=None,
                    actbox_name='', actbox_category='workflow', actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager',
                           'guard_expr':''},)
    t = wf.transitions.get('create_content')
    t.setProperties(title='Create content', new_state_id='work',
                    transition_behavior=(TRANSITION_ALLOWSUB_CREATE,),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='New',
                    actbox_category='',
                    actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; Member',
                           'guard_expr':''},)
