# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: SpontaneousMailFolderWorkflow.py,v 1.6 2004/05/07 12:18:20 anguenot Exp $

""" SpontaneousMailFolder type workflow definition

This file contains the main declaration for the SpontaneousMailFolder type.

See the requierments for further details. (And you'll need them).
Check the docs/design sub-directory you'll find a transition/state diagram.
"""

###########################################################################
#                             Notice for devs
#                            ------------------
#
#  - This workflow is driven by the MailFolderWfManager class.
#  - This class provides 3 differents things :
#    -> Methods dedicated to activate transitions
#    -> Methods for the transition guards to adjust permissions (DCWorkflow
#       based permissions are not enough
#    -> Variables containing list of users.
#  -  Assignment structure within sort of indexed dictionnary implementing
#       the assignement levels and the relations in between.
#  -  API dedicated to the Workflow.
#
###########################################################################

import os, sys
from zLOG import LOG, INFO, DEBUG
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.CPSCore.CPSWorkflow import \
     TRANSITION_INITIAL_CREATE, \
     TRANSITION_ALLOWSUB_CHECKOUT, \
     TRANSITION_ALLOWSUB_CREATE, \
     TRANSITION_ALLOWSUB_DELETE, \
     TRANSITION_ALLOWSUB_COPY, \
     TRANSITION_ALLOWSUB_MOVE, \
     TRANSITION_BEHAVIOR_PUBLISHING

from Products.DCWorkflow.Transitions import \
     TRIGGER_USER_ACTION

def spontaneousMailFolderWorkflowInstall(self):
    """Installs the workflow for the InComingMailFolder Type
    """

    portal = self.portal_url.getPortalObject()
    wftool = portal.portal_workflow

    wfids = wftool.objectIds()
    wfid = 'spontaneous_mail_folder_wf'

    if wfid in wfids:
        wftool.manage_delObjects([wfid])

    wftool.manage_addWorkflow(id=wfid,
                              workflow_type='cps_workflow (Web-configurable workflow for CPS)')

    wf = wftool[wfid]

    for p in (View, ModifyPortalContent, ):
        wf.addManagedPermission(p)

    ###########################################################################
    ###########################################################################

    #                                  STATES

    ###########################################################################
    ###########################################################################

    for s in ('writing',
              'validating',
              'closed',):
        wf.states.addState(s)


    ###########################################################################
    #                                   WRITING
    ###########################################################################

    s = wf.states.get('writing')

    s.setPermission(ModifyPortalContent, 0, ('Manager',
                                             'Owner',
                                             'WorkspaceManager'
                                             'WorkspaceMember',))
    s.setPermission(View, 0, ('Manager',
                              'Owner',
                              'WorkspaceManager',
                              'WorkspaceMember',
                              'WorkspaceReader',))

    s.setInitialState('writing')
    s.setProperties(title='The reply starts',
                    description='Here the reply starts and the next step will \
                    be the validation of the reply.',
                    transitions=('create_content',
                                 'cut_copy_paste',
                                 'reset',
                                 'modify',
                                 'comment',
                                 'close',
                                 'reset',
                                 'pre_assign',
                                 'validate',
                                 'reject_validate',))

    ###########################################################################
    #                               VALIDATING
    ###########################################################################

    s = wf.states.get('validating')

    s.setPermission(ModifyPortalContent, 0, ('Manager',
                                             'Owner',
                                             'WorkspaceManager',))
    s.setPermission(View, 0, ('Manager',
                              'Owner',
                              'WorkspaceManager',
                              'WorkspaceMember',
                              'WorkspaceReader',))

    s.setProperties(title='The mail folder is in an assigning \
    process(Dynamic)',
                    description='Mail Folder under an assigning process.\
                    3 lists to maintain corresponding to the 3 kind of actors\
                    (pilots/associates/watchers). \
                    Notice this process is dynamic.',
                    transitions=('create_content',
                                 'cut_copy_paste',
                                 'reset',
                                 'validate',
                                 'reject_validate',
                                 'close',
                                 'modify',
                                 'pre_assign',
                                 'comment',))


    ###########################################################################
    #                                      CLOSED
    ###########################################################################

    s = wf.states.get('closed')

    # Here to allow the pilots to re-open it.
    s.setPermission(View, 0, ('Manager',
                              'Owner',
                              'WorkspaceManager',))

    # XXX check who's able to see the MailFolder in a closed state
    s.setPermission(View, 0, ('Manager',
                              'Owner',
                              'WorkspaceManager',
                              'WorkspaceMember',
                              'WorkspaceReader'))

    s.setProperties(title='End of the story. Mail closed and reply done.',
                    description='Now the Manager/Pilots can publish either \
                    the whole folder or the mail reply.',
                    # 'copy_submit', not for the moment !
                    transitions=('unclose2validating',
                                 'unclose2writing',
                                 'copy_submit',
                                 'reset',))


    ###########################################################################
    ###########################################################################

    #                               TRANSITIONS

    ###########################################################################
    ###########################################################################

    for t in ('create',
              'copy_submit',
              'create_content',
              'cut_copy_paste',
              'pre_assign',
              'confirm_assign',
              'reset',
              'validate',
              'reject_validate',
              'modify', # silent one
              'close',
              'unclose2writing',
              'unclose2validating',
              'comment',):
        wf.transitions.addTransition(t)

    ###########################################################################
    #                                       CREATE
    ###########################################################################

    # Nota : this replaces the initial transition of DCWorkflow.
    t = wf.transitions.get('create')
    t.setProperties(title='Initial creation',
                    description='Intial transition like',
                    new_state_id='writing',
                    transition_behavior=(TRANSITION_INITIAL_CREATE, ),
                    clone_allowed_transitions=None,
                    actbox_name='', actbox_category='workflow', actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; Member',
                           'guard_expr':''},)

    ###########################################################################
    #                                     COPY SUBMIT
    ###########################################################################

    # Nota : unused right now : January 2004
    t = wf.transitions.get('copy_submit')
    t.setProperties(title='Copy content into a section for Publishing',
                    description='Will be allowed only when the mail folder \
                    will be closed',
                    new_state_id='',
                    transition_behavior=(TRANSITION_BEHAVIOR_PUBLISHING, ),
                    clone_allowed_transitions=('submit', 'publish'),
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_submit',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/content_submit_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager',
                           'guard_expr':'python:state_change["object"].getContent().canPublish(here)'},
                    )

    ###########################################################################
    #                                  CREATE CONTENT
    ###########################################################################

    t = wf.transitions.get('create_content')
    t.setProperties(title='Create content',
                    description='Allow sub Object Create',
                    new_state_id='writing',
                    transition_behavior=(TRANSITION_ALLOWSUB_CREATE,
                                         TRANSITION_ALLOWSUB_CHECKOUT,),
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='New',
                    actbox_category='',
                    actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member',
                           'guard_expr':''},)

    ###########################################################################
    #                                  CUT/COPY/PASTE
    ###########################################################################

    t = wf.transitions.get('cut_copy_paste')
    t.setProperties(title='Cut/Copy/Paste',
                    new_state_id='waiting',
                    transition_behavior=(TRANSITION_ALLOWSUB_DELETE,
                                         TRANSITION_ALLOWSUB_MOVE,
                                         TRANSITION_ALLOWSUB_COPY),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='New',
                    actbox_category='',
                actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member ',
                           'guard_expr':''},)

    ###########################################################################
    #                                  CONFIRM ASSIGN
    ###########################################################################

    t = wf.transitions.get('confirm_assign')
    t.setProperties(title='Confirm Assign the mail folder',
                    description='Confirm Assign the mail folder',
                    new_state_id='',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_mailfolder_validate_assign',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_validate_assign',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member',
                           'guard_expr':'python:state_change[\
                           "object"].getContent().canAssign(here)'},)

    ###########################################################################
    #                                  CLOSE
    ###########################################################################

    t = wf.transitions.get('close')
    t.setProperties(title='Close the mail folder.',
                    description='Ready for publication',
                    new_state_id='closed',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_close',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_close_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager;WorkspaceManager;Owner',
                           'guard_expr':'python:here.getContent().canClose(\
                           here)'},)

    ###########################################################################
    #                                  UNCLOSE2writing
    ###########################################################################

    t = wf.transitions.get('unclose2writing')
    t.setProperties(title='Un-close the mail folder.',
                    description='Let\'go back to the previous state',
                    new_state_id='writing',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_unclose',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_unclose_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager;WorkspaceManager',
                           'guard_expr':'python:here.getContent(\
                           ).canUnClose2writing(here)'},)

    ###########################################################################
    #                                  UNCLOSE2validating
    ###########################################################################

    t = wf.transitions.get('unclose2validating')
    t.setProperties(title='Un-close the mail folder.',
                    description='Let\'go back to the previous state',
                    new_state_id='validating',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_unclose',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_unclose_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager;WorkspaceManager',
                           'guard_expr':'python:here.getContent(\
                           ).canUnClose2validating(here)'},)

    ###########################################################################
    #                                  PRE ASSIGN
    ###########################################################################

    t = wf.transitions.get('pre_assign')
    t.setProperties(title='Pre assignment.',
                    description='The state remains the same.',
                    new_state_id='',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_mailfolder_affect',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_directory_template',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; WorkspaceReader; Member',
                           'guard_expr':''},)


    ###########################################################################
    #                                  RESET
    ###########################################################################

    t = wf.transitions.get('reset')
    t.setProperties(title='Reset and go back to waiting state.',
                    description='Reseting....',
                    new_state_id='writing',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_mailfolder_reinit',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_reinit_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member',
                           'guard_expr':'python:here.getContent(\
                           ).canReset(here)'},)

    ###########################################################################
    #                                       MODIFY
    ###########################################################################

    t = wf.transitions.get('modify')
    t.setProperties(title='Modification',
                    description='Silent one : modification',
                    new_state_id='',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    actbox_name='', actbox_category='workflow', actbox_url='',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; WorkspaceMember; Member ',
                           'guard_expr':''},)

    ###########################################################################
    #                                  VALIDATE
    ###########################################################################

    t = wf.transitions.get('validate')
    t.setProperties(title='Validates and then we go for a new dynamic \
    assignment',
                    description='Validating',
                    new_state_id='validating',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_validate',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_validate_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member',
                           'guard_expr':'python:here.getContent(\
                           ).canValidate(here)'},)

    ###########################################################################
    #                                  REJECT VALIDATE
    ###########################################################################

    t = wf.transitions.get('reject_validate')
    t.setProperties(title='Reject the validation',
                    description='Reject the validation',
                    new_state_id='',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_reject_validate',
                    actbox_category='workflow',
                    actbox_url='%(content_url)s/mailfolder_reject_validate_form',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Owner; Member',
                           'guard_expr':'python:here.getContent(\
                           ).canRejectValidate()'},)

    ###########################################################################
    #                                  COMMENT
    ###########################################################################

    t = wf.transitions.get('comment')
    t.setProperties(title='Add a comment',
                    description='Adding a comment',
                    new_state_id='',
                    transition_behavior=(),
                    clone_allowed_transitions=None,
                    trigger_type=TRIGGER_USER_ACTION,
                    actbox_name='action_mailfolder_comment',
                    actbox_category='',
                    actbox_url='%(content_url)s/mailfolder_comments',
                    props={'guard_permissions':'',
                           'guard_roles':'Manager; WorkspaceManager; \
                           WorkspaceMember; Member; WorkspaceReader',
                           'guard_expr':''},)

    ###########################################################################
    ###########################################################################

    #                                     VARIABLES

    ###########################################################################
    ###########################################################################

    for v in ('action',
              'actor',
              'comments',
              'review_history',
              'time',
              'dest_container',
              ):
        wf.variables.addVariable(v)


    wf.variables.setStateVar('review_state')
    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment',\
                       '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions':'',
                              'guard_roles':'Manager; WorkspaceManager; \
                              WorkspaceMember; WorkspaceReader; Member',
                              'guard_expr':''})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)

    vdef = wf.variables['dest_container']
    vdef.setProperties(description='Destination container for the last \
    paste/publish',
                       default_expr="python:state_change.kwargs.get(\
                       'dest_container', '')",
                       for_status=1, update_always=1)
