## Script (Python) "getFiltredMailList"
##parameters=filter=None, sorting_param='identifier', sort_order='desc'
#$Id: getFilteredMailList.py,v 1.3 2004/07/20 22:35:14 anguenot Exp $

"""Returns the list of mailfolders

filter correponds to a review_state.
sorting_param corresponds to the way the mailfolder list is sorted.
"""

import time

from zLOG import LOG, INFO

#
# MAILFOLDER EXISTING TYPES
# XXX PSIv2 (next version)
# Add the two new mailfolder types.
#
# XXX : baclport optimisation done for MISILL/PSIv2 (MisillMailManager)
#

MAIL_FOLDER_PORTAL_TYPES = ['InComingMailFolder', 'SpontaneousMailFolder']

#
# QUERY CATALOG IN HERE FOR MAILFOLDERS
#

path = context.portal_url.getRelativeContentPath(context)


start = time.time()
mail_list = context.search(query={'portal_type':MAIL_FOLDER_PORTAL_TYPES})

stop = time.time()

LOG("MailFolders search on : ",
    INFO,
    "Number of proxies %s" %str(len(mail_list)))
LOG("MailFolders search : ",
    INFO,
    str(stop - start))

mail_list = [x.getObject() for x in mail_list]

#
# FILTERING ON REVIEW_STATE AND/OR ACTOR ROLES
#

wftool = context.portal_workflow

start = time.time()

if filter == "all":
    mail_list = [x for x in mail_list
                 if wftool.getInfoFor(x, 'review_state') != "closed"]
elif filter == "to_handle":
    mail_list = [x for x in mail_list
                 if wftool.getInfoFor(x, 'review_state') not in['validating', 'closed']]
elif filter == "to_validate":
    mail_list = [x for x in mail_list
                 if wftool.getInfoFor(x, 'review_state') == "validating"]
elif filter == 'closed':
    mail_list = [x for x in mail_list
                 if wftool.getInfoFor(x, 'review_state') == "closed"]
elif filter == "to_work_on_it":
    mail_list = [x for x in mail_list
                 if 'Associate' in x.getContent().getCurrentMemberRoles()]
elif filter == "to_watch":
    mail_list = [x for x in mail_list
                 if 'Watcher' in x.getContent().getCurrentMemberRoles()]

stop = time.time()

LOG("MailFolders review_state filtering in : ",
    INFO,
    str(stop - start))

#
# CHECKING IF THE GUY IS A GLOBAL MANAGER
# THEN FILTERING A LITLE BIT MORE
#

if (not context.portal_membership.isAnonymousUser()) and filter not in ['all', 'closed']:
    authenticated = context.portal_membership.getAuthenticatedMember()
    isManager = authenticated.has_role(['Manager'], context)
    if isManager:
        res = []
        for proxy in mail_list:
            doc = proxy.getContent()
            assigned = doc.getAllAssignedMemberIds()
            assigned_ids = [x['login'] for x in assigned]
            if authenticated.getMemberId() in assigned_ids:
                res.append(proxy)
        mail_list = res
    # Now current_level only
    res = []
    for proxy in mail_list:
        doc = proxy.getContent()
        assigned = doc.getPilotMemberIds(level=doc.getCurrentLevel())
        if authenticated.getMemberId() in assigned:
            res.append(proxy)
    mail_list = res

#
# SORTING MAILFOLDER LIST AGAINST FILTERS
#

start = time.time()

if sorting_param is not None and sorting_param != 'identifier':
    mail_list = context.sortMailFolderProxyList(mail_list, sorting_param, sort_order)

stop = time.time()

LOG("MailFolders apply  filters in : ",
    INFO,
    str(stop - start))

return mail_list
