##parameters=criteria, value, sort_criteria=None, sort_order="asc", roles=None, people=None, result_type='both', add_all_users_group=None, pwd=None, member_id=None

# $Id: findUsers.py,v 1.3 2004/03/16 22:51:31 anguenot Exp $

"""Returns a list of users (one dict per user) which have 'value' in their 'criteria' field.
user means user or group if result_type == 'both'. result_type can be 'both', 'user', 'group'

If criteria is 'all', search in all attributes of the user.
If criteria is 'localroles', returns users that have a localrole on context.
If criteria is 'roles', returns users that have a localrole on context or inherited.
if criteria is 'contacts', returns contacts of current user.
if criteria is 'groups', returns groups of user (take id from value).
if criteria is 'groups_all', performs a search on all properties of groups
if criteria is 'ids', value is couple of two lists of ids of users and groups to be returned (just fill all fields)
if criteria is 'substitutes', returns substitutes of current user.

If roles is true, add context local role information in result dictionnaries
if add_all_users_group is true, add a role:Authenticated in results.

XXX ugly mess taken from misill
"""

pdirectories=context.portal_directories
directories=[pdirectories.members,] # the last has prority
criterias={
    'members' : ['sn', 'givenName', 'mail', 'id'],
    'groups' : ['group'],
    }

data_fields = {
    'members' : ['sn', 'givenName', 'mail', 'id'],
    'groups' : ['group'],
    }
#
# a space is added at the end of the name because zope cut
# field names on colon
#
all_users_group = ('role:Authenticated ' , {'sn' : ' Tout le monde',
                                            'id' : 'role:Authenticated ',
                                            'mail':'',
                                            'telephoneNumber':'',
                                            'description' : 'Tous les utilisateurs',
                                            'directory':'groups'})


def uniq(users):
    """Remove double users
    """

    dict={}
    for i in users:
        dict[i[0]] = i

    return dict.values()

def sort_users(users, criteria, order):
    """
    Sort users on criteria (can be 'name', or the id of one of the user's field).
    reverse sort if order=='desc'
    """
    users = uniq(users)
    users_tmp=None
    if criteria == 'name':
        users_tmp = [ ( (user[1].has_key('sn') and user[1].has_key('givenName')) \
                        and user[1]['sn'].lower()+user[1]['givenName'].lower() or \
                        (user[1].has_key('directory') and \
                         user[1]['directory'] == 'groups' and user[0].lower()) or "", user)
                      for user in users]
    else:
        users_tmp = [ ( (user[1].has_key(criteria) and user[1][criteria].lower() or ""), user)
                      for user in users]
    if users_tmp:
        users_tmp.sort()
        users = [user[1] for user in users_tmp]
        if order == 'desc':
            users.reverse()
    return users

def getRolesInfo(user):
    """Returns list of dictionnaries containing local role information about user
    """
    ri = roles_info[user]
    for i in ri:
        i['bc'] = context.getBreadCrumbs(url=i['url'],
                                         breadcrumb_set=getattr(context.REQUEST,
                                                                'breadcrumb_set', None))
    return ri


def addData(directory, user):
    """Add the directory ID and additionnal data about user in datamodel
    """
    if directory.id == 'groups' and groups_data.has_key(user[0]):
        user = (user[0], groups_data[user[0]])
    user[1]['directory'] = directory.id
    if not user[1].has_key('id'):
        user[1]['id'] = user[0]
    if roles:
        user[1]['roles'] = getRolesInfo(user[0].strip())
    return user

def search_groups(criteria, value):
    results = []
    groups_data = [{criteria:x} for x in pdirectories.groups.searchEntries(criteria=value)]
    if not value:
        for group in groups_data:
            group['directory'] = 'groups'
            results.append( (group[criteria], group) )

    if criteria in ['id']:
        for group in groups_data:
            if group[criteria].lower().find(value.lower()) != -1:
                groups_data[group[criteria]]['directory'] = 'groups'
                results.append( (group[criteria], group) )
    else:
        for group in groups_data:
            if group.has_key(criteria) and hasattr(group[criteria], 'lower') and \
                   group[criteria].lower().find(value.lower()) != -1:
                group['directory'] = 'groups'
                results.append( (group[criteria ], group) )
    return results

def search_users(directory, criteria, value):
    """Search for users
    """
    if roles:
        fields = data_fields[directory.id] + ['roles']
    else:
        fields = data_fields[directory.id]
    return [addData(directory, user)
            for user in directory.searchEntries(return_fields=fields, **{criteria: value})]

def getUsers(ids):
    """Create a list of dictionaries describing users from list of ids
    """
    results = []
    for id in ids:
        for dir in directories:
            try:
                user = dir.getEntry(id)
            except:
                continue
            if user:
                results.append(addData(dir, (id, user)))
                break
    return results

def getGroups(ids):
    """Create a list of dictionaries describing group from list of ids
    """
    results = []
    group=None
    for id in ids:
        if id == "role:Authenticated":
            results.append(addData(pdirectories.groups, all_users_group))
        else:
            try:
                group = pdirectories.groups.getEntry(id)
            except:
                pass
            if group:
                results.append(addData(pdirectories.groups, (id, group)))
                group=None
    return results

def getContacts():
    """Return contacts of current user
    """
    prop = "contacts"
    mtool = context.portal_membership
    member = mtool.getAuthenticatedMember()
    isAnon = mtool.isAnonymousUser()
    directories = pdirectories.objectIds()

    if isAnon:
        raise "Unauthorized", "You must be authenticated to have contacts"

    uname = member.getUserName();

    my_folder = context.portal_url.getPortalObject()[context.PersoRoot].members
    my_folder = getattr(my_folder,uname)
    my_folder = my_folder.getContent()

    if my_folder.hasProperty(prop):
        contacts = my_folder[prop]
    else:
        contacts = []

    result=[]
    for entry in contacts:
        if entry:
            tmp = entry.split(':', 1)
            directory = tmp[0]
            id = tmp[1]

            if directory in directories:
                dir = getattr(pdirectories, directory)
                if dir.hasEntry(id):
                    user = dir.getEntry(id)
                    result.append(addData(dir, (id, user)))
    return result

def getSubstitutes():
    """Returns the substitutes for a given member
    """

    prop = "substitutes"
    mtool = context.portal_membership
    member = mtool.getAuthenticatedMember()
    isAnon = mtool.isAnonymousUser()
    directories = pdirectories.objectIds()

    if isAnon:
        raise "Unauthorized", "You must be authenticated to have contacts"

    if member_id is None:
        uname = member.getUserName()
    else:
        uname = member_id

    my_folder = context.portal_url.getPortalObject()[context.PersoRoot].members
    my_folder = getattr(my_folder,uname)
    my_folder = my_folder.getContent()

    if my_folder.hasProperty(prop):
        substitutes = my_folder[prop]
    else:
        substitutes = []

    result=[]
    for entry in substitutes:
        if entry:
            tmp = entry.split(':', 1)
            directory = tmp[0]
            id = tmp[1]

            if directory in directories:
                dir = getattr(pdirectories, directory)
                if dir.hasEntry(id):
                    user = dir.getEntry(id)
                    result.append(addData(dir, (id, user)))
    return result

#
# CORE
#

results = []
temp=[]

if criteria == 'groups':
    result_type = 'group'
    results += search_users(pdirectories.groups, 'members', [value])

elif criteria == 'groups_all':
    criteria = 'all'
    result_type = 'group'

if result_type in ['group', 'both']:
    if criteria == "all" and value:
        for c in criterias['groups']:
            results += search_groups(c, value)
    else:
        results += search_groups(criteria, value)

    if add_all_users_group:
        results.append(all_users_group)

if result_type != 'group':

    if criteria == 'contacts':
        results = getContacts()
    elif criteria == 'substitutes':
        results = getSubstitutes()
    elif criteria == "roles":
        localroles_info = context.getCPSLocalRoles()
        roles_info = {}
        users = []
        groups = []
        for user in localroles_info[0].keys():
            if user.startswith('user:'):
                roles_info[user[5:]] = localroles_info[0][user]
                users.append(user[5:])
            elif user.startswith('group:'):
                roles_info[user[6:]] = localroles_info[0][user]
                groups.append(user[6:])
        results = getUsers(users) + getGroups(groups)

    elif criteria == "localroles":
        results = getUsers(context.get_local_roles_by_users().keys()) + \
                  getGroups(context.get_local_roles_by_users(groups = 1).keys())

    elif criteria == "ids":
        results = getUsers(value[0])
        results += getGroups(value[1])

    else:
        if not criteria:
            return []
        if criteria == "all" and not value:
            criteria = 'id'
        for directory in directories:
            if criteria == "all":
                for c in criterias[directory.id]:
                    results += search_users(directory, c, value)
            else:
                results += search_users(directory, criteria, value)

    contact={}


    if people==1:
        contact=context.getContent().getContact()

        for i in contact.keys():
            results.append((i,contact[i]))

if sort_criteria:
    results = sort_users(results, sort_criteria, sort_order)

return results
