# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 CGEY <http://cgey.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Julien Anguenot <ja@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Notifications.py,v 1.2 2004/05/07 12:04:46 anguenot Exp $

__author__ = "Julien Anguenot <ja@nuxeo.com>"

from zLOG import LOG, DEBUG, INFO

from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View

"""Notifications

Will be use within the WorkflowManager class.
Mostly, to notify assignements. (Pilots/Associates/watchers)

Quite primitif since CPSSubscriptions wasn't ready at that time.
"""

class Notifications:
    """Notifications

    Will be use within the WorkflowManager class.
    Mostly, to notify assignements. (Pilots/Associates/watchers)

    Quite primitif since CPSSubscriptions wasn't ready at that time.
    """

    security = ClassSecurityInfo()

    def _getMailNotificationRuleObject(self):
        """Returns the id of the MailNotificationRule object
        in the MailRootFolder
        """

        root_mail_folder = self.getRootMailFolder(proxy=1) #skins
        subscriptions_folder = getattr(
            root_mail_folder,
            self.portal_subscriptions.getSubscriptionContainerId())
        return getattr(
            subscriptions_folder,
            self.portal_subscriptions. getMailNotificationRuleObjectId())

    security.declareProtected(View, 'notify_mailfolder_event')
    def notify_mailfolder_event(self, mailfolder, event_id, member_ids):
        """Notify a given event type related to Mail Folder workflow life cycle
        """

        emails = []
        for member_id in member_ids:
            emails.append(self.getMemberEmail(member_id))

        ob = self._getMailNotificationRuleObject()
        if ob is not None:
            if emails is not None:
                try:
                    ob.notifyRecipients(event_id,
                                        mailfolder,
                                        emails=emails)
                    LOG("notify_mailfolder_event()",
                        INFO,
                        emails)
                except (TypeError):
                    #
                    # When the mail is not well formed
                    # We don't know if there's a control on the mail fomat
                    #
                    LOG("notify_mailfolder_event()",
                        INFO,
                        'Problems sending mail to : ',
                        emails)
        else:
            LOG("MisillFolderNotifications : notify_mailfolder_event()",
                INFO,
                'ERROR : The mail notification object is not present \
                within the root mailfolder',
                'Impossible send any mails')
