# Copyright (c) 2004 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2004 capgemini <http://capgemini.com>
# Copyright (c) 2004 Ministre de L'intrieur (MISILL)
#               <http://www.interieur.gouv.fr/>
# Author: Jrme SAUTRET <jerome.sautret@capgemini.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: WorkflowImportExport.py,v 1.1 2004/06/14 16:46:47 anguenot Exp $

__author__ = "Jrme SAUTRET <jerome.sautret@capgemini.com>"

"""Workflow Import/Export functionalities
 """

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent


class WorkflowImportExport:
    """Base MailFolder Class
    """

    security = ClassSecurityInfo()

    # property name of user folder used to store saved workflows
    my_workflows_prop = "my_workflows" 
    my_workflows_global_folder = "courriers"
    
    ##############################################################
    #  My Workflows methods
    ##############################################################

    def _my_workflow_get(self, is_global=0):
        """
        return the list containing my workflows and the object where it is stored
        """
        portal = self.portal_url.getPortalObject()
        mtool = portal.portal_membership
        member = mtool.getAuthenticatedMember()
        isAnon = mtool.isAnonymousUser()

        if isAnon:
            raise "Unauthorized", "You must be authenticated to use My Workflow"

        if is_global == 0:
            
            uname = member.getUserName();

            folder = portal[portal.PersoRoot].members
            folder = getattr(folder,uname)
            folder = folder.getContent()
        else:
            folder = getattr(portal, self.my_workflows_global_folder)
            
        if not hasattr(folder, self.my_workflows_prop):
            setattr(folder, self.my_workflows_prop, [])
        my_workflows = getattr(folder, self.my_workflows_prop)

        return (my_workflows, folder)

    def _my_workflow_set(self, my_workflows, folder):
        """
        set my_workflows into the correct property
        """
        setattr(folder, self.my_workflows_prop, my_workflows)

        
    security.declareProtected(ModifyPortalContent, "my_workflow_load")
    def my_workflow_load(self, id=None, is_global=0, url_from=None, proxy=None, REQUEST=None):
        """
        Load the workflow and set it on context document.
        If global == 0, load it from the my_workflows home folder property,
        else load it from my_workflows courier folder.
        """
        if id is None:
            if REQUEST is not None:
                psm="?portal_status_message=Vous+devez+slectionner+un+workflow+dans+la+dernire+colonne"
                if url_from is not None:
                    REQUEST['RESPONSE'].redirect(url_from+psm)
                else:
                    REQUEST['RESPONSE'].redirect(self.absolute_url()+psm)
            return

        my_workflows, folder = self._my_workflow_get(is_global=is_global)
        saved_workflow = my_workflows[id]
        saved_conf = saved_workflow['conf']
        for pilot in saved_conf[1]:
            self._insertPilots(left=pilot['level'], right=pilot['level'],
                               id=pilot['login'], context=proxy)
        self.doPreAssign(www=saved_conf[0], context=proxy)
        

        if REQUEST is not None:
            psm="?portal_status_message=Le+workflow++%s+++" % saved_workflow['title'] +\
                 "t+appliqu+au+courrier+courant" 
            if url_from is not None:
                REQUEST['RESPONSE'].redirect(url_from+psm)
            else:
                REQUEST['RESPONSE'].redirect(self.absolute_url()+psm)


    security.declareProtected(View, "my_workflow_save")
    def my_workflow_save(self, title, description, is_global=0, url_from=None, REQUEST=None):
        """
        Save the workflow currently set on context document.
        If global == 0, save it on the my_workflows home folder property,
        else save it on my_workflows courier folder.
        """

        if not title:
            if REQUEST is not None:
                psm="/my_workflow_save_form?portal_status_message=Veuillez+spcifier+un+titre"
                if url_from is not None:
                    REQUEST['RESPONSE'].redirect(url_from+psm)
                else:
                    REQUEST['RESPONSE'].redirect(self.absolute_url()+psm)
            return

        
        if is_global:
            portal = self.portal_url.getPortalObject()
            checkPermission = portal.portal_membership.checkPermission
            if not checkPermission('Manage portal', self):
                raise Unauthorized('You must have "Manage portal" permission to do that')

        # get current worflow configuration
        conf = (self.getAllPreAssigned(),
                [pilot for pilot in self.getAllAssigned()
                 if pilot['role'] == 'pilote'] # and pilot['level'] != 0]
                )

        my_workflow_new = {'title': title, 'description': description, 'conf': conf}

        my_workflows, folder = self._my_workflow_get(is_global=is_global)

        my_workflows.append(my_workflow_new)

        self._my_workflow_set(my_workflows, folder)

        if REQUEST is not None:
            psm="?portal_status_message=Le+workflow++%s+++t+sauvegard" % title
            if url_from is not None:
                REQUEST['RESPONSE'].redirect(url_from+psm)
            else:
                REQUEST['RESPONSE'].redirect(self.absolute_url()+psm)
        
    security.declareProtected(View, "my_workflow_get")
    def my_workflow_get(self, is_global=0):
        """
        return a list of dicts describing the workflows already saved 
        """
        my_workflows, folder = self._my_workflow_get(is_global=is_global)

        return my_workflows

    security.declareProtected(View, "my_workflows_delete")
    def my_workflows_delete(self, ids=[], is_global=0, url_from=None, REQUEST=None):
        """
        remove the saved workflows whose index are in ids
        """
        my_workflows, folder = self._my_workflow_get(is_global=is_global)
        ids.reverse()
        for index in ids:
            del my_workflows[index]

        self._my_workflow_set(my_workflows, folder)

        if REQUEST is not None:
            nb=len(ids)
            if nb == 0:
                psm="?portal_status_message=Aucun+workflow+n'a+t+slctionn"
            elif nb == 1:
                psm="?portal_status_message=Le+workflow+a+t+effac"
            else:
                psm="?portal_status_message=Les+workflows+ont+t+effacs"
            if url_from is not None:
                REQUEST['RESPONSE'].redirect(url_from+psm)
            else:
                REQUEST['RESPONSE'].redirect(self.absolute_url()+psm)
    ###################################################################
    ###################################################################
